# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.order_line_item_discount_type import OrderLineItemDiscountType
from .money import MoneyParams
from ..types.order_line_item_discount_scope import OrderLineItemDiscountScope


class OrderLineItemDiscountParams(typing_extensions.TypedDict):
    """
    Represents a discount that applies to one or more line items in an
    order.

    Fixed-amount, order-scoped discounts are distributed across all non-zero line item totals.
    The amount distributed to each line item is relative to the
    amount contributed by the item to the order subtotal.
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID that identifies the discount only within this order.
    """

    catalog_object_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The catalog object ID referencing [CatalogDiscount](entity:CatalogDiscount).
    """

    catalog_version: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The version of the catalog object that this discount references.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The discount's name.
    """

    type: typing_extensions.NotRequired[OrderLineItemDiscountType]
    """
    The type of the discount.
    
    Discounts that do not reference a catalog object ID must have a type of
    `FIXED_PERCENTAGE` or `FIXED_AMOUNT`.
    See [OrderLineItemDiscountType](#type-orderlineitemdiscounttype) for possible values
    """

    percentage: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The percentage of the discount, as a string representation of a decimal number.
    A value of `7.25` corresponds to a percentage of 7.25%.
    
    `percentage` is not set for amount-based discounts.
    """

    amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total declared monetary amount of the discount.
    
    `amount_money` is not set for percentage-based discounts.
    """

    applied_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of discount actually applied to the line item.
    
    The amount represents the amount of money applied as a line-item scoped discount.
    When an amount-based discount is scoped to the entire order, the value
    of `applied_money` is different than `amount_money` because the total
    amount of the discount is distributed across all line items.
    """

    metadata: typing_extensions.NotRequired[typing.Optional[typing.Dict[str, typing.Optional[str]]]]
    """
    Application-defined data attached to this discount. Metadata fields are intended
    to store descriptive references or associations with an entity in another system or store brief
    information about the object. Square does not process this field; it only stores and returns it
    in relevant API calls. Do not use metadata to store any sensitive information (such as personally
    identifiable information or card details).
    
    Keys written by applications must be 60 characters or less and must be in the character set
    `[a-zA-Z0-9_-]`. Entries can also include metadata generated by Square. These keys are prefixed
    with a namespace, separated from the key with a ':' character.
    
    Values have a maximum length of 255 characters.
    
    An application can have up to 10 entries per metadata field.
    
    Entries written by applications are private and can only be read or modified by the same
    application.
    
    For more information, see [Metadata](https://developer.squareup.com/docs/build-basics/metadata).
    """

    scope: typing_extensions.NotRequired[OrderLineItemDiscountScope]
    """
    Indicates the level at which the discount applies. For `ORDER` scoped discounts,
    Square generates references in `applied_discounts` on all order line items that do
    not have them. For `LINE_ITEM` scoped discounts, the discount only applies to line items
    with a discount reference in their `applied_discounts` field.
    
    This field is immutable. To change the scope of a discount, you must delete
    the discount and re-add it as a new discount.
    See [OrderLineItemDiscountScope](#type-orderlineitemdiscountscope) for possible values
    """

    reward_ids: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    The reward IDs corresponding to this discount. The application and
    specification of discounts that have `reward_ids` are completely controlled by the backing
    criteria corresponding to the reward tiers of the rewards that are added to the order
    through the Loyalty API. To manually unapply discounts that are the result of added rewards,
    the rewards must be removed from the order through the Loyalty API.
    """

    pricing_rule_id: typing_extensions.NotRequired[str]
    """
    The object ID of a [pricing rule](entity:CatalogPricingRule) to be applied
    automatically to this discount. The specification and application of the discounts, to
    which a `pricing_rule_id` is assigned, are completely controlled by the corresponding
    pricing rule.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .subscription import SubscriptionParams


class CreateSubscriptionResponseParams(typing_extensions.TypedDict):
    """
    Defines output parameters in a response from the
    [CreateSubscription](api-endpoint:Subscriptions-CreateSubscription) endpoint.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors encountered during the request.
    """

    subscription: typing_extensions.NotRequired[SubscriptionParams]
    """
    The newly created subscription.
    
    For more information, see
    [Subscription object](https://developer.squareup.com/docs/subscriptions-api/manage-subscriptions#subscription-object).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.catalog_custom_attribute_definition_type import CatalogCustomAttributeDefinitionType


class CatalogCustomAttributeValueParams(typing_extensions.TypedDict):
    """
    An instance of a custom attribute. Custom attributes can be defined and
    added to `ITEM` and `ITEM_VARIATION` type catalog objects.
    [Read more about custom attributes](https://developer.squareup.com/docs/catalog-api/add-custom-attributes).
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the custom attribute.
    """

    string_value: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The string value of the custom attribute.  Populated if `type` = `STRING`.
    """

    custom_attribute_definition_id: typing_extensions.NotRequired[str]
    """
    The id of the [CatalogCustomAttributeDefinition](entity:CatalogCustomAttributeDefinition) this value belongs to.
    """

    type: typing_extensions.NotRequired[CatalogCustomAttributeDefinitionType]
    """
    A copy of type from the associated `CatalogCustomAttributeDefinition`.
    See [CatalogCustomAttributeDefinitionType](#type-catalogcustomattributedefinitiontype) for possible values
    """

    number_value: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Populated if `type` = `NUMBER`. Contains a string
    representation of a decimal number, using a `.` as the decimal separator.
    """

    boolean_value: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    A `true` or `false` value. Populated if `type` = `BOOLEAN`.
    """

    selection_uid_values: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    One or more choices from `allowed_selections`. Populated if `type` = `SELECTION`.
    """

    key: typing_extensions.NotRequired[str]
    """
    If the associated `CatalogCustomAttributeDefinition` object is defined by another application, this key is prefixed by the defining application ID.
    For example, if the CatalogCustomAttributeDefinition has a key attribute of "cocoa_brand" and the defining application ID is "abcd1234", this key is "abcd1234:cocoa_brand"
    when the application making the request is different from the application defining the custom attribute definition. Otherwise, the key is simply "cocoa_brand".
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .customer import CustomerParams


class ListCustomersResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the `ListCustomers` endpoint.

    Either `errors` or `customers` is present in a given response (never both).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    customers: typing_extensions.NotRequired[typing.Sequence[CustomerParams]]
    """
    The customer profiles associated with the Square account or an empty object (`{}`) if none are found.
    Only customer profiles with public information (`given_name`, `family_name`, `company_name`, `email_address`, or
    `phone_number`) are included in the response.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    A pagination cursor to retrieve the next set of results for the
    original query. A cursor is only present if the request succeeded and additional results
    are available.
    
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    count: typing_extensions.NotRequired[int]
    """
    The total count of customers associated with the Square account. Only customer profiles with public information
    (`given_name`, `family_name`, `company_name`, `email_address`, or `phone_number`) are counted. This field is present
    only if `count` is set to `true` in the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.team_member_assigned_locations_assignment_type import TeamMemberAssignedLocationsAssignmentType
import typing


class TeamMemberAssignedLocationsParams(typing_extensions.TypedDict):
    """
    An object that represents a team member's assignment to locations.
    """

    assignment_type: typing_extensions.NotRequired[TeamMemberAssignedLocationsAssignmentType]
    """
    The current assignment type of the team member.
    See [TeamMemberAssignedLocationsAssignmentType](#type-teammemberassignedlocationsassignmenttype) for possible values
    """

    location_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The explicit locations that the team member is assigned to.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .checkout_options import CheckoutOptionsParams
from .pre_populated_data import PrePopulatedDataParams


class PaymentLinkParams(typing_extensions.TypedDict):
    id: typing_extensions.NotRequired[str]
    """
    The Square-assigned ID of the payment link.
    """

    version: int
    """
    The Square-assigned version number, which is incremented each time an update is committed to the payment link.
    """

    description: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The optional description of the `payment_link` object.
    It is primarily for use by your application and is not used anywhere.
    """

    order_id: typing_extensions.NotRequired[str]
    """
    The ID of the order associated with the payment link.
    """

    checkout_options: typing_extensions.NotRequired[CheckoutOptionsParams]
    """
    The checkout options configured for the payment link.
    For more information, see [Optional Checkout Configurations](https://developer.squareup.com/docs/checkout-api/optional-checkout-configurations).
    """

    pre_populated_data: typing_extensions.NotRequired[PrePopulatedDataParams]
    """
    Describes buyer data to prepopulate
    on the checkout page.
    """

    url: typing_extensions.NotRequired[str]
    """
    The shortened URL of the payment link.
    """

    long_url: typing_extensions.NotRequired[str]
    """
    The long URL of the payment link.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the payment link was created, in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the payment link was last updated, in RFC 3339 format.
    """

    payment_note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    An optional note. After Square processes the payment, this note is added to the
    resulting `Payment`.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .payment import PaymentParams


class CancelPaymentResponseParams(typing_extensions.TypedDict):
    """
    Defines the response returned by [CancelPayment](api-endpoint:Payments-CancelPayment).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """

    payment: typing_extensions.NotRequired[PaymentParams]
    """
    The successfully canceled `Payment` object.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .search_team_members_filter import SearchTeamMembersFilterParams


class SearchTeamMembersQueryParams(typing_extensions.TypedDict):
    """
    Represents the parameters in a search for `TeamMember` objects.
    """

    filter: typing_extensions.NotRequired[SearchTeamMembersFilterParams]
    """
    The options to filter by.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .delete_order_custom_attribute_response import DeleteOrderCustomAttributeResponseParams


class BulkDeleteOrderCustomAttributesResponseParams(typing_extensions.TypedDict):
    """
    Represents a response from deleting one or more order custom attributes.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    values: typing.Dict[str, DeleteOrderCustomAttributeResponseParams]
    """
     A map of responses that correspond to individual delete requests. Each response has the same ID 
    as the corresponding request and contains either a `custom_attribute` or an `errors` field.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class TipSettingsParams(typing_extensions.TypedDict):
    allow_tipping: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether tipping is enabled for this checkout. Defaults to false.
    """

    separate_tip_screen: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether tip options should be presented on the screen before presenting
    the signature screen during card payment. Defaults to false.
    """

    custom_tip_field: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether custom tip amounts are allowed during the checkout flow. Defaults to false.
    """

    tip_percentages: typing_extensions.NotRequired[typing.Optional[typing.Sequence[int]]]
    """
    A list of tip percentages that should be presented during the checkout flow, specified as
    up to 3 non-negative integers from 0 to 100 (inclusive). Defaults to 15, 20, and 25.
    """

    smart_tipping: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Enables the "Smart Tip Amounts" behavior.
    Exact tipping options depend on the region in which the Square seller is active.
    
    For payments under 10.00, in the Australia, Canada, Ireland, United Kingdom, and United States, tipping options are presented as no tip, .50, 1.00 or 2.00.
    
    For payment amounts of 10.00 or greater, tipping options are presented as the following percentages: 0%, 5%, 10%, 15%.
    
    If set to true, the `tip_percentages` settings is ignored.
    Defaults to false.
    
    To learn more about smart tipping, see [Accept Tips with the Square App](https://squareup.com/help/us/en/article/5069-accept-tips-with-the-square-app).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.catalog_item_food_and_beverage_details_dietary_preference_type import (
    CatalogItemFoodAndBeverageDetailsDietaryPreferenceType,
)
from ..types.catalog_item_food_and_beverage_details_ingredient_standard_ingredient import (
    CatalogItemFoodAndBeverageDetailsIngredientStandardIngredient,
)
import typing


class CatalogItemFoodAndBeverageDetailsIngredientParams(typing_extensions.TypedDict):
    """
    Describes the ingredient used in a `FOOD_AND_BEV` item.
    """

    type: typing_extensions.NotRequired[CatalogItemFoodAndBeverageDetailsDietaryPreferenceType]
    """
    The dietary preference type of the ingredient. Supported values include `STANDARD` and `CUSTOM` as specified in `FoodAndBeverageDetails.DietaryPreferenceType`.
    See [DietaryPreferenceType](#type-dietarypreferencetype) for possible values
    """

    standard_name: typing_extensions.NotRequired[CatalogItemFoodAndBeverageDetailsIngredientStandardIngredient]
    """
    The name of the ingredient from a standard pre-defined list. This should be null if it's a custom dietary preference.
    See [StandardIngredient](#type-standardingredient) for possible values
    """

    custom_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of a custom user-defined ingredient. This should be null if it's a standard dietary preference.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .loyalty_promotion_incentive import LoyaltyPromotionIncentive
from .loyalty_promotion_available_time_data import LoyaltyPromotionAvailableTimeData
from .loyalty_promotion_trigger_limit import LoyaltyPromotionTriggerLimit
from .loyalty_promotion_status import LoyaltyPromotionStatus
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyPromotion(UncheckedBaseModel):
    """
    Represents a promotion for a [loyalty program](entity:LoyaltyProgram). Loyalty promotions enable buyers
    to earn extra points on top of those earned from the base program.

    A loyalty program can have a maximum of 10 loyalty promotions with an `ACTIVE` or `SCHEDULED` status.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-assigned ID of the promotion.
    """

    name: str = pydantic.Field()
    """
    The name of the promotion.
    """

    incentive: LoyaltyPromotionIncentive = pydantic.Field()
    """
    The points incentive for the promotion. This field defines whether promotion points
    are earned by multiplying base program points or by adding a specified number of points.
    """

    available_time: LoyaltyPromotionAvailableTimeData = pydantic.Field()
    """
    The scheduling information that defines when purchases can qualify to earn points from an `ACTIVE` promotion.
    """

    trigger_limit: typing.Optional[LoyaltyPromotionTriggerLimit] = pydantic.Field(default=None)
    """
    The number of times a buyer can earn promotion points during a specified interval.
    If not specified, buyers can trigger the promotion an unlimited number of times.
    """

    status: typing.Optional[LoyaltyPromotionStatus] = pydantic.Field(default=None)
    """
    The current status of the promotion.
    See [LoyaltyPromotionStatus](#type-loyaltypromotionstatus) for possible values
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the promotion was created, in RFC 3339 format.
    """

    canceled_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the promotion was canceled, in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the promotion was last updated, in RFC 3339 format.
    """

    loyalty_program_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the [loyalty program](entity:LoyaltyProgram) associated with the promotion.
    """

    minimum_spend_amount_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The minimum purchase amount required to earn promotion points. If specified, this amount is positive.
    """

    qualifying_item_variation_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The IDs of any qualifying `ITEM_VARIATION` [catalog objects](entity:CatalogObject). If specified,
    the purchase must include at least one of these items to qualify for the promotion.
    
    This option is valid only if the base loyalty program uses a `VISIT` or `SPEND` accrual rule.
    With `SPEND` accrual rules, make sure that qualifying promotional items are not excluded.
    
    You can specify `qualifying_item_variation_ids` or `qualifying_category_ids` for a given promotion, but not both.
    """

    qualifying_category_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The IDs of any qualifying `CATEGORY` [catalog objects](entity:CatalogObject). If specified,
    the purchase must include at least one item from one of these categories to qualify for the promotion.
    
    This option is valid only if the base loyalty program uses a `VISIT` or `SPEND` accrual rule.
    With `SPEND` accrual rules, make sure that qualifying promotional items are not excluded.
    
    You can specify `qualifying_category_ids` or `qualifying_item_variation_ids` for a promotion, but not both.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .availability import AvailabilityParams
from .error import ErrorParams


class SearchAvailabilityResponseParams(typing_extensions.TypedDict):
    availabilities: typing_extensions.NotRequired[typing.Sequence[AvailabilityParams]]
    """
    List of appointment slots available for booking.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawCheckoutClient
from .payment_links.client import PaymentLinksClient
from ..core.request_options import RequestOptions
from ..types.retrieve_location_settings_response import RetrieveLocationSettingsResponse
from ..requests.checkout_location_settings import CheckoutLocationSettingsParams
from ..types.update_location_settings_response import UpdateLocationSettingsResponse
from ..types.retrieve_merchant_settings_response import RetrieveMerchantSettingsResponse
from ..requests.checkout_merchant_settings import CheckoutMerchantSettingsParams
from ..types.update_merchant_settings_response import UpdateMerchantSettingsResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawCheckoutClient
from .payment_links.client import AsyncPaymentLinksClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CheckoutClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCheckoutClient(client_wrapper=client_wrapper)
        self.payment_links = PaymentLinksClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCheckoutClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCheckoutClient
        """
        return self._raw_client

    def retrieve_location_settings(
        self, location_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveLocationSettingsResponse:
        """
        Retrieves the location-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        location_id : str
            The ID of the location for which to retrieve settings.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveLocationSettingsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.checkout.retrieve_location_settings(
            location_id="location_id",
        )
        """
        response = self._raw_client.retrieve_location_settings(location_id, request_options=request_options)
        return response.data

    def update_location_settings(
        self,
        location_id: str,
        *,
        location_settings: CheckoutLocationSettingsParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateLocationSettingsResponse:
        """
        Updates the location-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        location_id : str
            The ID of the location for which to retrieve settings.

        location_settings : CheckoutLocationSettingsParams
            Describe your updates using the `location_settings` object. Make sure it contains only the fields that have changed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateLocationSettingsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.checkout.update_location_settings(
            location_id="location_id",
            location_settings={},
        )
        """
        response = self._raw_client.update_location_settings(
            location_id, location_settings=location_settings, request_options=request_options
        )
        return response.data

    def retrieve_merchant_settings(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveMerchantSettingsResponse:
        """
        Retrieves the merchant-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveMerchantSettingsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.checkout.retrieve_merchant_settings()
        """
        response = self._raw_client.retrieve_merchant_settings(request_options=request_options)
        return response.data

    def update_merchant_settings(
        self,
        *,
        merchant_settings: CheckoutMerchantSettingsParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateMerchantSettingsResponse:
        """
        Updates the merchant-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        merchant_settings : CheckoutMerchantSettingsParams
            Describe your updates using the `merchant_settings` object. Make sure it contains only the fields that have changed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateMerchantSettingsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.checkout.update_merchant_settings(
            merchant_settings={},
        )
        """
        response = self._raw_client.update_merchant_settings(
            merchant_settings=merchant_settings, request_options=request_options
        )
        return response.data


class AsyncCheckoutClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCheckoutClient(client_wrapper=client_wrapper)
        self.payment_links = AsyncPaymentLinksClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCheckoutClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCheckoutClient
        """
        return self._raw_client

    async def retrieve_location_settings(
        self, location_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveLocationSettingsResponse:
        """
        Retrieves the location-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        location_id : str
            The ID of the location for which to retrieve settings.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveLocationSettingsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.checkout.retrieve_location_settings(
                location_id="location_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.retrieve_location_settings(location_id, request_options=request_options)
        return response.data

    async def update_location_settings(
        self,
        location_id: str,
        *,
        location_settings: CheckoutLocationSettingsParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateLocationSettingsResponse:
        """
        Updates the location-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        location_id : str
            The ID of the location for which to retrieve settings.

        location_settings : CheckoutLocationSettingsParams
            Describe your updates using the `location_settings` object. Make sure it contains only the fields that have changed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateLocationSettingsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.checkout.update_location_settings(
                location_id="location_id",
                location_settings={},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update_location_settings(
            location_id, location_settings=location_settings, request_options=request_options
        )
        return response.data

    async def retrieve_merchant_settings(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveMerchantSettingsResponse:
        """
        Retrieves the merchant-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveMerchantSettingsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.checkout.retrieve_merchant_settings()


        asyncio.run(main())
        """
        response = await self._raw_client.retrieve_merchant_settings(request_options=request_options)
        return response.data

    async def update_merchant_settings(
        self,
        *,
        merchant_settings: CheckoutMerchantSettingsParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateMerchantSettingsResponse:
        """
        Updates the merchant-level settings for a Square-hosted checkout page.

        Parameters
        ----------
        merchant_settings : CheckoutMerchantSettingsParams
            Describe your updates using the `merchant_settings` object. Make sure it contains only the fields that have changed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateMerchantSettingsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.checkout.update_merchant_settings(
                merchant_settings={},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update_merchant_settings(
            merchant_settings=merchant_settings, request_options=request_options
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CatalogInfoResponseLimitsParams(typing_extensions.TypedDict):
    batch_upsert_max_objects_per_batch: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum number of objects that may appear within a single batch in a
    `/v2/catalog/batch-upsert` request.
    """

    batch_upsert_max_total_objects: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum number of objects that may appear across all batches in a
    `/v2/catalog/batch-upsert` request.
    """

    batch_retrieve_max_object_ids: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum number of object IDs that may appear in a `/v2/catalog/batch-retrieve`
    request.
    """

    search_max_page_limit: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum number of results that may be returned in a page of a
    `/v2/catalog/search` response.
    """

    batch_delete_max_object_ids: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum number of object IDs that may be included in a single
    `/v2/catalog/batch-delete` request.
    """

    update_item_taxes_max_item_ids: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum number of item IDs that may be included in a single
    `/v2/catalog/update-item-taxes` request.
    """

    update_item_taxes_max_taxes_to_enable: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum number of tax IDs to be enabled that may be included in a single
    `/v2/catalog/update-item-taxes` request.
    """

    update_item_taxes_max_taxes_to_disable: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum number of tax IDs to be disabled that may be included in a single
    `/v2/catalog/update-item-taxes` request.
    """

    update_item_modifier_lists_max_item_ids: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum number of item IDs that may be included in a single
    `/v2/catalog/update-item-modifier-lists` request.
    """

    update_item_modifier_lists_max_modifier_lists_to_enable: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum number of modifier list IDs to be enabled that may be included in
    a single `/v2/catalog/update-item-modifier-lists` request.
    """

    update_item_modifier_lists_max_modifier_lists_to_disable: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum number of modifier list IDs to be disabled that may be included in
    a single `/v2/catalog/update-item-modifier-lists` request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .team_member_status import TeamMemberStatus
from .team_member_assigned_locations import TeamMemberAssignedLocations
from .wage_setting import WageSetting
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TeamMember(UncheckedBaseModel):
    """
    A record representing an individual team member for a business.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique ID for the team member.
    """

    reference_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A second ID used to associate the team member with an entity in another system.
    """

    is_owner: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the team member is the owner of the Square account.
    """

    status: typing.Optional[TeamMemberStatus] = pydantic.Field(default=None)
    """
    Describes the status of the team member.
    See [TeamMemberStatus](#type-teammemberstatus) for possible values
    """

    given_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The given name (that is, the first name) associated with the team member.
    """

    family_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The family name (that is, the last name) associated with the team member.
    """

    email_address: typing.Optional[str] = pydantic.Field(default=None)
    """
    The email address associated with the team member. After accepting the invitation
    from Square, only the team member can change this value.
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The team member's phone number, in E.164 format. For example:
    +14155552671 - the country code is 1 for US
    +551155256325 - the country code is 55 for BR
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the team member was created, in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the team member was last updated, in RFC 3339 format.
    """

    assigned_locations: typing.Optional[TeamMemberAssignedLocations] = pydantic.Field(default=None)
    """
    Describes the team member's assigned locations.
    """

    wage_setting: typing.Optional[WageSetting] = pydantic.Field(default=None)
    """
    Information about the team member's overtime exemption status, job assignments, and compensation.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .date_range import DateRange
import pydantic
from .scheduled_shift_workday_matcher import ScheduledShiftWorkdayMatcher
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ScheduledShiftWorkday(UncheckedBaseModel):
    """
    A `ScheduledShift` search query filter parameter that sets a range of days that
    a `Shift` must start or end in before passing the filter condition.
    """

    date_range: typing.Optional[DateRange] = pydantic.Field(default=None)
    """
    Dates for fetching the scheduled shifts.
    """

    match_scheduled_shifts_by: typing.Optional[ScheduledShiftWorkdayMatcher] = pydantic.Field(default=None)
    """
    The strategy on which the dates are applied.
    See [ScheduledShiftWorkdayMatcher](#type-scheduledshiftworkdaymatcher) for possible values
    """

    default_timezone: typing.Optional[str] = pydantic.Field(default=None)
    """
    Location-specific timezones convert workdays to datetime filters.
    Every location included in the query must have a timezone or this field
    must be provided as a fallback. Format: the IANA timezone database
    identifier for the relevant timezone.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .catalog_object import CatalogObjectParams
from .catalog_id_mapping import CatalogIdMappingParams


class BatchUpsertCatalogObjectsResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    objects: typing_extensions.NotRequired[typing.Sequence[CatalogObjectParams]]
    """
    The created successfully created CatalogObjects.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The database [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates) of this update in RFC 3339 format, e.g., "2016-09-04T23:59:33.123Z".
    """

    id_mappings: typing_extensions.NotRequired[typing.Sequence[CatalogIdMappingParams]]
    """
    The mapping between client and server IDs for this upsert.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CashAppDetailsParams(typing_extensions.TypedDict):
    """
    Additional details about `WALLET` type payments with the `brand` of `CASH_APP`.
    """

    buyer_full_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the Cash App account holder.
    """

    buyer_country_code: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The country of the Cash App account holder, in ISO 3166-1-alpha-2 format.
    
    For possible values, see [Country](entity:Country).
    """

    buyer_cashtag: typing_extensions.NotRequired[str]
    """
    $Cashtag of the Cash App account holder.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .customer_filter import CustomerFilterParams
from .customer_sort import CustomerSortParams


class CustomerQueryParams(typing_extensions.TypedDict):
    """
    Represents filtering and sorting criteria for a [SearchCustomers](api-endpoint:Customers-SearchCustomers) request.
    """

    filter: typing_extensions.NotRequired[CustomerFilterParams]
    """
    The filtering criteria for the search query. A query can contain multiple filters in any combination.
    Multiple filters are combined as `AND` statements.
    
    __Note:__ Combining multiple filters as `OR` statements is not supported. Instead, send multiple single-filter
    searches and join the result sets.
    """

    sort: typing_extensions.NotRequired[CustomerSortParams]
    """
    Sorting criteria for query results. The default behavior is to sort 
    customers alphabetically by `given_name` and `family_name`.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

BookingBookingSource = typing.Union[
    typing.Literal["FIRST_PARTY_MERCHANT", "FIRST_PARTY_BUYER", "THIRD_PARTY_BUYER", "API"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CheckoutLocationSettingsPolicy(UncheckedBaseModel):
    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique ID to identify the policy when making changes. You must set the UID for policy updates, but it’s optional when setting new policies.
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    The title of the policy. This is required when setting the description, though you can update it in a different request.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the policy.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .sort_order import SortOrder
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogQuerySortedAttribute(UncheckedBaseModel):
    """
    The query expression to specify the key to sort search results.
    """

    attribute_name: str = pydantic.Field()
    """
    The attribute whose value is used as the sort key.
    """

    initial_attribute_value: typing.Optional[str] = pydantic.Field(default=None)
    """
    The first attribute value to be returned by the query. Ascending sorts will return only
    objects with this value or greater, while descending sorts will return only objects with this value
    or less. If unset, start at the beginning (for ascending sorts) or end (for descending sorts).
    """

    sort_order: typing.Optional[SortOrder] = pydantic.Field(default=None)
    """
    The desired sort order, `"ASC"` (ascending) or `"DESC"` (descending).
    See [SortOrder](#type-sortorder) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogItemOptionValueForItemVariation(UncheckedBaseModel):
    """
    A `CatalogItemOptionValue` links an item variation to an item option as
    an item option value. For example, a t-shirt item may offer a color option and
    a size option. An item option value would represent each variation of t-shirt:
    For example, "Color:Red, Size:Small" or "Color:Blue, Size:Medium".
    """

    item_option_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique id of an item option.
    """

    item_option_value_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique id of the selected value for the item option.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams


class ShippingFeeParams(typing_extensions.TypedDict):
    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name for the shipping fee.
    """

    charge: MoneyParams
    """
    The amount and currency for the shipping fee.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing_extensions
import typing
from ..core.serialization import FieldMetadata
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class FilterValue(UncheckedBaseModel):
    """
    A filter to select resources based on an exact field value. For any given
    value, the value can only be in one property. Depending on the field, either
    all properties can be set or only a subset will be available.

    Refer to the documentation of the field.
    """

    all_: typing_extensions.Annotated[typing.Optional[typing.List[str]], FieldMetadata(alias="all")] = pydantic.Field(
        default=None
    )
    """
    A list of terms that must be present on the field of the resource.
    """

    any: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of terms where at least one of them must be present on the
    field of the resource.
    """

    none: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of terms that must not be present on the field the resource
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class DeviceComponentDetailsMeasurementParams(typing_extensions.TypedDict):
    """
    A value qualified by unit of measure.
    """

    value: typing_extensions.NotRequired[typing.Optional[int]]



# This file was auto-generated by Fern from our API Definition.

import typing

ComponentComponentType = typing.Union[
    typing.Literal["APPLICATION", "CARD_READER", "BATTERY", "WIFI", "ETHERNET", "PRINTER"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

ErrorCode = typing.Union[
    typing.Literal[
        "INTERNAL_SERVER_ERROR",
        "UNAUTHORIZED",
        "ACCESS_TOKEN_EXPIRED",
        "ACCESS_TOKEN_REVOKED",
        "CLIENT_DISABLED",
        "FORBIDDEN",
        "INSUFFICIENT_SCOPES",
        "APPLICATION_DISABLED",
        "V1_APPLICATION",
        "V1_ACCESS_TOKEN",
        "CARD_PROCESSING_NOT_ENABLED",
        "MERCHANT_SUBSCRIPTION_NOT_FOUND",
        "BAD_REQUEST",
        "MISSING_REQUIRED_PARAMETER",
        "INCORRECT_TYPE",
        "INVALID_TIME",
        "INVALID_TIME_RANGE",
        "INVALID_VALUE",
        "INVALID_CURSOR",
        "UNKNOWN_QUERY_PARAMETER",
        "CONFLICTING_PARAMETERS",
        "EXPECTED_JSON_BODY",
        "INVALID_SORT_ORDER",
        "VALUE_REGEX_MISMATCH",
        "VALUE_TOO_SHORT",
        "VALUE_TOO_LONG",
        "VALUE_TOO_LOW",
        "VALUE_TOO_HIGH",
        "VALUE_EMPTY",
        "ARRAY_LENGTH_TOO_LONG",
        "ARRAY_LENGTH_TOO_SHORT",
        "ARRAY_EMPTY",
        "EXPECTED_BOOLEAN",
        "EXPECTED_INTEGER",
        "EXPECTED_FLOAT",
        "EXPECTED_STRING",
        "EXPECTED_OBJECT",
        "EXPECTED_ARRAY",
        "EXPECTED_MAP",
        "EXPECTED_BASE64_ENCODED_BYTE_ARRAY",
        "INVALID_ARRAY_VALUE",
        "INVALID_ENUM_VALUE",
        "INVALID_CONTENT_TYPE",
        "INVALID_FORM_VALUE",
        "CUSTOMER_NOT_FOUND",
        "ONE_INSTRUMENT_EXPECTED",
        "NO_FIELDS_SET",
        "TOO_MANY_MAP_ENTRIES",
        "MAP_KEY_LENGTH_TOO_SHORT",
        "MAP_KEY_LENGTH_TOO_LONG",
        "CUSTOMER_MISSING_NAME",
        "CUSTOMER_MISSING_EMAIL",
        "INVALID_PAUSE_LENGTH",
        "INVALID_DATE",
        "UNSUPPORTED_COUNTRY",
        "UNSUPPORTED_CURRENCY",
        "APPLE_TTP_PIN_TOKEN",
        "CARD_EXPIRED",
        "INVALID_EXPIRATION",
        "INVALID_EXPIRATION_YEAR",
        "INVALID_EXPIRATION_DATE",
        "UNSUPPORTED_CARD_BRAND",
        "UNSUPPORTED_ENTRY_METHOD",
        "INVALID_ENCRYPTED_CARD",
        "INVALID_CARD",
        "PAYMENT_AMOUNT_MISMATCH",
        "GENERIC_DECLINE",
        "CVV_FAILURE",
        "ADDRESS_VERIFICATION_FAILURE",
        "INVALID_ACCOUNT",
        "CURRENCY_MISMATCH",
        "INSUFFICIENT_FUNDS",
        "INSUFFICIENT_PERMISSIONS",
        "CARDHOLDER_INSUFFICIENT_PERMISSIONS",
        "INVALID_LOCATION",
        "TRANSACTION_LIMIT",
        "VOICE_FAILURE",
        "PAN_FAILURE",
        "EXPIRATION_FAILURE",
        "CARD_NOT_SUPPORTED",
        "READER_DECLINED",
        "INVALID_PIN",
        "MISSING_PIN",
        "MISSING_ACCOUNT_TYPE",
        "INVALID_POSTAL_CODE",
        "INVALID_FEES",
        "MANUALLY_ENTERED_PAYMENT_NOT_SUPPORTED",
        "PAYMENT_LIMIT_EXCEEDED",
        "GIFT_CARD_AVAILABLE_AMOUNT",
        "ACCOUNT_UNUSABLE",
        "BUYER_REFUSED_PAYMENT",
        "DELAYED_TRANSACTION_EXPIRED",
        "DELAYED_TRANSACTION_CANCELED",
        "DELAYED_TRANSACTION_CAPTURED",
        "DELAYED_TRANSACTION_FAILED",
        "CARD_TOKEN_EXPIRED",
        "CARD_TOKEN_USED",
        "AMOUNT_TOO_HIGH",
        "UNSUPPORTED_INSTRUMENT_TYPE",
        "REFUND_AMOUNT_INVALID",
        "REFUND_ALREADY_PENDING",
        "PAYMENT_NOT_REFUNDABLE",
        "PAYMENT_NOT_REFUNDABLE_DUE_TO_DISPUTE",
        "REFUND_ERROR_PAYMENT_NEEDS_COMPLETION",
        "REFUND_DECLINED",
        "INSUFFICIENT_PERMISSIONS_FOR_REFUND",
        "INVALID_CARD_DATA",
        "SOURCE_USED",
        "SOURCE_EXPIRED",
        "UNSUPPORTED_LOYALTY_REWARD_TIER",
        "LOCATION_MISMATCH",
        "ORDER_UNPAID_NOT_RETURNABLE",
        "IDEMPOTENCY_KEY_REUSED",
        "UNEXPECTED_VALUE",
        "SANDBOX_NOT_SUPPORTED",
        "INVALID_EMAIL_ADDRESS",
        "INVALID_PHONE_NUMBER",
        "CHECKOUT_EXPIRED",
        "BAD_CERTIFICATE",
        "INVALID_SQUARE_VERSION_FORMAT",
        "API_VERSION_INCOMPATIBLE",
        "CARD_PRESENCE_REQUIRED",
        "UNSUPPORTED_SOURCE_TYPE",
        "CARD_MISMATCH",
        "PLAID_ERROR",
        "PLAID_ERROR_ITEM_LOGIN_REQUIRED",
        "PLAID_ERROR_RATE_LIMIT",
        "CARD_DECLINED",
        "VERIFY_CVV_FAILURE",
        "VERIFY_AVS_FAILURE",
        "CARD_DECLINED_CALL_ISSUER",
        "CARD_DECLINED_VERIFICATION_REQUIRED",
        "BAD_EXPIRATION",
        "CHIP_INSERTION_REQUIRED",
        "ALLOWABLE_PIN_TRIES_EXCEEDED",
        "RESERVATION_DECLINED",
        "UNKNOWN_BODY_PARAMETER",
        "NOT_FOUND",
        "APPLE_PAYMENT_PROCESSING_CERTIFICATE_HASH_NOT_FOUND",
        "METHOD_NOT_ALLOWED",
        "NOT_ACCEPTABLE",
        "REQUEST_TIMEOUT",
        "CONFLICT",
        "GONE",
        "REQUEST_ENTITY_TOO_LARGE",
        "UNSUPPORTED_MEDIA_TYPE",
        "UNPROCESSABLE_ENTITY",
        "RATE_LIMITED",
        "NOT_IMPLEMENTED",
        "BAD_GATEWAY",
        "SERVICE_UNAVAILABLE",
        "TEMPORARY_ERROR",
        "GATEWAY_TIMEOUT",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .money import MoneyParams
import typing
from ..types.payout_fee_type import PayoutFeeType


class PayoutFeeParams(typing_extensions.TypedDict):
    """
    Represents a payout fee that can incur as part of a payout.
    """

    amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The money amount of the payout fee.
    """

    effective_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The timestamp of when the fee takes effect, in RFC 3339 format.
    """

    type: typing_extensions.NotRequired[PayoutFeeType]
    """
    The type of fee assessed as part of the payout.
    See [PayoutFeeType](#type-payoutfeetype) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .tender_buy_now_pay_later_details_brand import TenderBuyNowPayLaterDetailsBrand
import pydantic
from .tender_buy_now_pay_later_details_status import TenderBuyNowPayLaterDetailsStatus
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TenderBuyNowPayLaterDetails(UncheckedBaseModel):
    """
    Represents the details of a tender with `type` `BUY_NOW_PAY_LATER`.
    """

    buy_now_pay_later_brand: typing.Optional[TenderBuyNowPayLaterDetailsBrand] = pydantic.Field(default=None)
    """
    The Buy Now Pay Later brand.
    See [Brand](#type-brand) for possible values
    """

    status: typing.Optional[TenderBuyNowPayLaterDetailsStatus] = pydantic.Field(default=None)
    """
    The buy now pay later payment's current state (such as `AUTHORIZED` or
    `CAPTURED`). See [TenderBuyNowPayLaterDetailsStatus](entity:TenderBuyNowPayLaterDetailsStatus)
    for possible values.
    See [Status](#type-status) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawCheckoutsClient
from ...requests.terminal_checkout import TerminalCheckoutParams
from ...core.request_options import RequestOptions
from ...types.create_terminal_checkout_response import CreateTerminalCheckoutResponse
from ...requests.terminal_checkout_query import TerminalCheckoutQueryParams
from ...types.search_terminal_checkouts_response import SearchTerminalCheckoutsResponse
from ...types.get_terminal_checkout_response import GetTerminalCheckoutResponse
from ...types.cancel_terminal_checkout_response import CancelTerminalCheckoutResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawCheckoutsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CheckoutsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCheckoutsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCheckoutsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCheckoutsClient
        """
        return self._raw_client

    def create(
        self,
        *,
        idempotency_key: str,
        checkout: TerminalCheckoutParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateTerminalCheckoutResponse:
        """
        Creates a Terminal checkout request and sends it to the specified device to take a payment
        for the requested amount.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this `CreateCheckout` request. Keys can be any valid string but
            must be unique for every `CreateCheckout` request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        checkout : TerminalCheckoutParams
            The checkout to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateTerminalCheckoutResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.terminal.checkouts.create(
            idempotency_key="28a0c3bc-7839-11ea-bc55-0242ac130003",
            checkout={
                "amount_money": {"amount": 2610, "currency": "USD"},
                "reference_id": "id11572",
                "note": "A brief note",
                "device_options": {"device_id": "dbb5d83a-7838-11ea-bc55-0242ac130003"},
            },
        )
        """
        response = self._raw_client.create(
            idempotency_key=idempotency_key, checkout=checkout, request_options=request_options
        )
        return response.data

    def search(
        self,
        *,
        query: typing.Optional[TerminalCheckoutQueryParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchTerminalCheckoutsResponse:
        """
        Returns a filtered list of Terminal checkout requests created by the application making the request. Only Terminal checkout requests created for the merchant scoped to the OAuth token are returned. Terminal checkout requests are available for 30 days.

        Parameters
        ----------
        query : typing.Optional[TerminalCheckoutQueryParams]
            Queries Terminal checkouts based on given conditions and the sort order.
            Leaving these unset returns all checkouts with the default sort order.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more information.

        limit : typing.Optional[int]
            Limits the number of results returned for a single request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchTerminalCheckoutsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.terminal.checkouts.search(
            query={"filter": {"status": "COMPLETED"}},
            limit=2,
        )
        """
        response = self._raw_client.search(query=query, cursor=cursor, limit=limit, request_options=request_options)
        return response.data

    def get(
        self, checkout_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTerminalCheckoutResponse:
        """
        Retrieves a Terminal checkout request by `checkout_id`. Terminal checkout requests are available for 30 days.

        Parameters
        ----------
        checkout_id : str
            The unique ID for the desired `TerminalCheckout`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTerminalCheckoutResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.terminal.checkouts.get(
            checkout_id="checkout_id",
        )
        """
        response = self._raw_client.get(checkout_id, request_options=request_options)
        return response.data

    def cancel(
        self, checkout_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CancelTerminalCheckoutResponse:
        """
        Cancels a Terminal checkout request if the status of the request permits it.

        Parameters
        ----------
        checkout_id : str
            The unique ID for the desired `TerminalCheckout`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelTerminalCheckoutResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.terminal.checkouts.cancel(
            checkout_id="checkout_id",
        )
        """
        response = self._raw_client.cancel(checkout_id, request_options=request_options)
        return response.data


class AsyncCheckoutsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCheckoutsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCheckoutsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCheckoutsClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        idempotency_key: str,
        checkout: TerminalCheckoutParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateTerminalCheckoutResponse:
        """
        Creates a Terminal checkout request and sends it to the specified device to take a payment
        for the requested amount.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this `CreateCheckout` request. Keys can be any valid string but
            must be unique for every `CreateCheckout` request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        checkout : TerminalCheckoutParams
            The checkout to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateTerminalCheckoutResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.terminal.checkouts.create(
                idempotency_key="28a0c3bc-7839-11ea-bc55-0242ac130003",
                checkout={
                    "amount_money": {"amount": 2610, "currency": "USD"},
                    "reference_id": "id11572",
                    "note": "A brief note",
                    "device_options": {
                        "device_id": "dbb5d83a-7838-11ea-bc55-0242ac130003"
                    },
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            idempotency_key=idempotency_key, checkout=checkout, request_options=request_options
        )
        return response.data

    async def search(
        self,
        *,
        query: typing.Optional[TerminalCheckoutQueryParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchTerminalCheckoutsResponse:
        """
        Returns a filtered list of Terminal checkout requests created by the application making the request. Only Terminal checkout requests created for the merchant scoped to the OAuth token are returned. Terminal checkout requests are available for 30 days.

        Parameters
        ----------
        query : typing.Optional[TerminalCheckoutQueryParams]
            Queries Terminal checkouts based on given conditions and the sort order.
            Leaving these unset returns all checkouts with the default sort order.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more information.

        limit : typing.Optional[int]
            Limits the number of results returned for a single request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchTerminalCheckoutsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.terminal.checkouts.search(
                query={"filter": {"status": "COMPLETED"}},
                limit=2,
            )


        asyncio.run(main())
        """
        response = await self._raw_client.search(
            query=query, cursor=cursor, limit=limit, request_options=request_options
        )
        return response.data

    async def get(
        self, checkout_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTerminalCheckoutResponse:
        """
        Retrieves a Terminal checkout request by `checkout_id`. Terminal checkout requests are available for 30 days.

        Parameters
        ----------
        checkout_id : str
            The unique ID for the desired `TerminalCheckout`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTerminalCheckoutResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.terminal.checkouts.get(
                checkout_id="checkout_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(checkout_id, request_options=request_options)
        return response.data

    async def cancel(
        self, checkout_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CancelTerminalCheckoutResponse:
        """
        Cancels a Terminal checkout request if the status of the request permits it.

        Parameters
        ----------
        checkout_id : str
            The unique ID for the desired `TerminalCheckout`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelTerminalCheckoutResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.terminal.checkouts.cancel(
                checkout_id="checkout_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.cancel(checkout_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .customer_text_filter import CustomerTextFilterParams
from ..types.country import Country


class CustomerAddressFilterParams(typing_extensions.TypedDict):
    """
    The customer address filter. This filter is used in a [CustomerCustomAttributeFilterValue](entity:CustomerCustomAttributeFilterValue) filter when
    searching by an `Address`-type custom attribute.
    """

    postal_code: typing_extensions.NotRequired[CustomerTextFilterParams]
    """
    The postal code to search for. Only an `exact` match is supported.
    """

    country: typing_extensions.NotRequired[Country]
    """
    The country code to search for.
    See [Country](#type-country) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .money import MoneyParams
import typing
from ..types.action_cancel_reason import ActionCancelReason


class TerminalRefundParams(typing_extensions.TypedDict):
    """
    Represents a payment refund processed by the Square Terminal. Only supports Interac (Canadian debit network) payment refunds.
    """

    id: typing_extensions.NotRequired[str]
    """
    A unique ID for this `TerminalRefund`.
    """

    refund_id: typing_extensions.NotRequired[str]
    """
    The reference to the payment refund created by completing this `TerminalRefund`.
    """

    payment_id: str
    """
    The unique ID of the payment being refunded.
    """

    order_id: typing_extensions.NotRequired[str]
    """
    The reference to the Square order ID for the payment identified by the `payment_id`.
    """

    amount_money: MoneyParams
    """
    The amount of money, inclusive of `tax_money`, that the `TerminalRefund` should return.
    This value is limited to the amount taken in the original payment minus any completed or
    pending refunds.
    """

    reason: str
    """
    A description of the reason for the refund.
    """

    device_id: str
    """
    The unique ID of the device intended for this `TerminalRefund`.
    The Id can be retrieved from /v2/devices api.
    """

    deadline_duration: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The RFC 3339 duration, after which the refund is automatically canceled.
    A `TerminalRefund` that is `PENDING` is automatically `CANCELED` and has a cancellation reason
    of `TIMED_OUT`.
    
    Default: 5 minutes from creation.
    
    Maximum: 5 minutes
    """

    status: typing_extensions.NotRequired[str]
    """
    The status of the `TerminalRefund`.
    Options: `PENDING`, `IN_PROGRESS`, `CANCEL_REQUESTED`, `CANCELED`, or `COMPLETED`.
    """

    cancel_reason: typing_extensions.NotRequired[ActionCancelReason]
    """
    Present if the status is `CANCELED`.
    See [ActionCancelReason](#type-actioncancelreason) for possible values
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The time when the `TerminalRefund` was created, as an RFC 3339 timestamp.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The time when the `TerminalRefund` was last updated, as an RFC 3339 timestamp.
    """

    app_id: typing_extensions.NotRequired[str]
    """
    The ID of the application that created the refund.
    """

    location_id: typing_extensions.NotRequired[str]
    """
    The location of the device where the `TerminalRefund` was directed.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .money import MoneyParams


class ChargeRequestAdditionalRecipientParams(typing_extensions.TypedDict):
    """
    Represents an additional recipient (other than the merchant) entitled to a portion of the tender.
    Support is currently limited to USD, CAD and GBP currencies
    """

    location_id: str
    """
    The location ID for a recipient (other than the merchant) receiving a portion of the tender.
    """

    description: str
    """
    The description of the additional recipient.
    """

    amount_money: MoneyParams
    """
    The amount of money distributed to the recipient.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .v1order_history_entry_action import V1OrderHistoryEntryAction
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class V1OrderHistoryEntry(UncheckedBaseModel):
    """
    V1OrderHistoryEntry
    """

    action: typing.Optional[V1OrderHistoryEntryAction] = pydantic.Field(default=None)
    """
    The type of action performed on the order.
    See [V1OrderHistoryEntryAction](#type-v1orderhistoryentryaction) for possible values
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The time when the action was performed, in ISO 8601 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .custom_attribute import CustomAttribute
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BulkUpsertOrderCustomAttributesRequestUpsertCustomAttribute(UncheckedBaseModel):
    """
    Represents one upsert within the bulk operation.
    """

    custom_attribute: CustomAttribute = pydantic.Field()
    """
    The custom attribute to create or update, with the following fields:
    
    - `value`. This value must conform to the `schema` specified by the definition. 
    For more information, see [Value data types](https://developer.squareup.com/docs/customer-custom-attributes-api/custom-attributes#value-data-types).
    
    - `version`. To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
    control, include this optional field and specify the current version of the custom attribute.
    """

    idempotency_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique identifier for this request, used to ensure idempotency. 
    For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).
    """

    order_id: str = pydantic.Field()
    """
    The ID of the target [order](entity:Order).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
import typing_extensions
from ..core.serialization import FieldMetadata
from .custom_attribute_definition_visibility import CustomAttributeDefinitionVisibility
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CustomAttributeDefinition(UncheckedBaseModel):
    """
    Represents a definition for custom attribute values. A custom attribute definition
    specifies the key, visibility, schema, and other properties for a custom attribute.
    """

    key: typing.Optional[str] = pydantic.Field(default=None)
    """
    The identifier
    of the custom attribute definition and its corresponding custom attributes. This value
    can be a simple key, which is the key that is provided when the custom attribute definition
    is created, or a qualified key, if the requesting
    application is not the definition owner. The qualified key consists of the application ID
    of the custom attribute definition owner
    followed by the simple key that was provided when the definition was created. It has the
    format application_id:simple key.
    
    The value for a simple key can contain up to 60 alphanumeric characters, periods (.),
    underscores (_), and hyphens (-).
    
    This field can not be changed
    after the custom attribute definition is created. This field is required when creating
    a definition and must be unique per application, seller, and resource type.
    """

    schema_: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="schema")
    ] = pydantic.Field(default=None)
    """
    The JSON schema for the custom attribute definition, which determines the data type of the corresponding custom attributes. For more information,
    see [Custom Attributes Overview](https://developer.squareup.com/docs/devtools/customattributes/overview). This field is required when creating a definition.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the custom attribute definition for API and seller-facing UI purposes. The name must
    be unique within the seller and application pair. This field is required if the
    `visibility` field is `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Seller-oriented description of the custom attribute definition, including any constraints
    that the seller should observe. May be displayed as a tooltip in Square UIs. This field is
    required if the `visibility` field is `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.
    """

    visibility: typing.Optional[CustomAttributeDefinitionVisibility] = pydantic.Field(default=None)
    """
    Specifies how the custom attribute definition and its values should be shared with
    the seller and other applications. If no value is specified, the value defaults to `VISIBILITY_HIDDEN`.
    See [Visibility](#type-visibility) for possible values
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    Read only. The current version of the custom attribute definition.
    The value is incremented each time the custom attribute definition is updated.
    When updating a custom attribute definition, you can provide this field
    and specify the current version of the custom attribute definition to enable
    [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency).
    
    On writes, this field must be set to the latest version. Stale writes are rejected.
    
    This field can also be used to enforce strong consistency for reads. For more information about strong consistency for reads,
    see [Custom Attributes Overview](https://developer.squareup.com/docs/devtools/customattributes/overview).
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp that indicates when the custom attribute definition was created or most recently updated,
    in RFC 3339 format.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp that indicates when the custom attribute definition was created, in RFC 3339 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
import typing
from .catalog_object_image import CatalogObjectImage
from .catalog_object_category import CatalogObjectCategory
from .catalog_object_item_variation import CatalogObjectItemVariation
from .catalog_object_tax import CatalogObjectTax
from .catalog_object_discount import CatalogObjectDiscount
from .catalog_object_modifier import CatalogObjectModifier
from .catalog_object_pricing_rule import CatalogObjectPricingRule
from .catalog_object_product_set import CatalogObjectProductSet
from .catalog_object_time_period import CatalogObjectTimePeriod
from .catalog_object_measurement_unit import CatalogObjectMeasurementUnit
from .catalog_object_subscription_plan_variation import CatalogObjectSubscriptionPlanVariation
from .catalog_object_item_option_value import CatalogObjectItemOptionValue
from .catalog_object_custom_attribute_definition import CatalogObjectCustomAttributeDefinition
from .catalog_object_quick_amounts_settings import CatalogObjectQuickAmountsSettings
from .catalog_object_availability_period import CatalogObjectAvailabilityPeriod
import typing

if typing.TYPE_CHECKING:
    from .catalog_object_item import CatalogObjectItem
    from .catalog_object_modifier_list import CatalogObjectModifierList
    from .catalog_object_item_option import CatalogObjectItemOption
    from .catalog_object_subscription_plan import CatalogObjectSubscriptionPlan
CatalogObject = typing.Union[
    "CatalogObjectItem",
    CatalogObjectImage,
    CatalogObjectCategory,
    CatalogObjectItemVariation,
    CatalogObjectTax,
    CatalogObjectDiscount,
    "CatalogObjectModifierList",
    CatalogObjectModifier,
    CatalogObjectPricingRule,
    CatalogObjectProductSet,
    CatalogObjectTimePeriod,
    CatalogObjectMeasurementUnit,
    CatalogObjectSubscriptionPlanVariation,
    "CatalogObjectItemOption",
    CatalogObjectItemOptionValue,
    CatalogObjectCustomAttributeDefinition,
    CatalogObjectQuickAmountsSettings,
    "CatalogObjectSubscriptionPlan",
    CatalogObjectAvailabilityPeriod,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .catalog_query_sorted_attribute import CatalogQuerySortedAttributeParams
from .catalog_query_exact import CatalogQueryExactParams
from .catalog_query_set import CatalogQuerySetParams
from .catalog_query_prefix import CatalogQueryPrefixParams
from .catalog_query_range import CatalogQueryRangeParams
from .catalog_query_text import CatalogQueryTextParams
from .catalog_query_items_for_tax import CatalogQueryItemsForTaxParams
from .catalog_query_items_for_modifier_list import CatalogQueryItemsForModifierListParams
from .catalog_query_items_for_item_options import CatalogQueryItemsForItemOptionsParams
from .catalog_query_item_variations_for_item_option_values import CatalogQueryItemVariationsForItemOptionValuesParams


class CatalogQueryParams(typing_extensions.TypedDict):
    """
    A query composed of one or more different types of filters to narrow the scope of targeted objects when calling the `SearchCatalogObjects` endpoint.

    Although a query can have multiple filters, only certain query types can be combined per call to [SearchCatalogObjects](api-endpoint:Catalog-SearchCatalogObjects).
    Any combination of the following types may be used together:
    - [exact_query](entity:CatalogQueryExact)
    - [prefix_query](entity:CatalogQueryPrefix)
    - [range_query](entity:CatalogQueryRange)
    - [sorted_attribute_query](entity:CatalogQuerySortedAttribute)
    - [text_query](entity:CatalogQueryText)

    All other query types cannot be combined with any others.

    When a query filter is based on an attribute, the attribute must be searchable.
    Searchable attributes are listed as follows, along their parent types that can be searched for with applicable query filters.

    Searchable attribute and objects queryable by searchable attributes:
    - `name`:  `CatalogItem`, `CatalogItemVariation`, `CatalogCategory`, `CatalogTax`, `CatalogDiscount`, `CatalogModifier`, `CatalogModifierList`, `CatalogItemOption`, `CatalogItemOptionValue`
    - `description`: `CatalogItem`, `CatalogItemOptionValue`
    - `abbreviation`: `CatalogItem`
    - `upc`: `CatalogItemVariation`
    - `sku`: `CatalogItemVariation`
    - `caption`: `CatalogImage`
    - `display_name`: `CatalogItemOption`

    For example, to search for [CatalogItem](entity:CatalogItem) objects by searchable attributes, you can use
    the `"name"`, `"description"`, or `"abbreviation"` attribute in an applicable query filter.
    """

    sorted_attribute_query: typing_extensions.NotRequired[CatalogQuerySortedAttributeParams]
    """
    A query expression to sort returned query result by the given attribute.
    """

    exact_query: typing_extensions.NotRequired[CatalogQueryExactParams]
    """
    An exact query expression to return objects with attribute name and value
    matching the specified attribute name and value exactly. Value matching is case insensitive.
    """

    set_query: typing_extensions.NotRequired[CatalogQuerySetParams]
    """
    A set query expression to return objects with attribute name and value
    matching the specified attribute name and any of the specified attribute values exactly.
    Value matching is case insensitive.
    """

    prefix_query: typing_extensions.NotRequired[CatalogQueryPrefixParams]
    """
    A prefix query expression to return objects with attribute values
    that have a prefix matching the specified string value. Value matching is case insensitive.
    """

    range_query: typing_extensions.NotRequired[CatalogQueryRangeParams]
    """
    A range query expression to return objects with numeric values
    that lie in the specified range.
    """

    text_query: typing_extensions.NotRequired[CatalogQueryTextParams]
    """
    A text query expression to return objects whose searchable attributes contain all of the given
    keywords, irrespective of their order. For example, if a `CatalogItem` contains custom attribute values of
    `{"name": "t-shirt"}` and `{"description": "Small, Purple"}`, the query filter of `{"keywords": ["shirt", "sma", "purp"]}`
    returns this item.
    """

    items_for_tax_query: typing_extensions.NotRequired[CatalogQueryItemsForTaxParams]
    """
    A query expression to return items that have any of the specified taxes (as identified by the corresponding `CatalogTax` object IDs) enabled.
    """

    items_for_modifier_list_query: typing_extensions.NotRequired[CatalogQueryItemsForModifierListParams]
    """
    A query expression to return items that have any of the given modifier list (as identified by the corresponding `CatalogModifierList`s IDs) enabled.
    """

    items_for_item_options_query: typing_extensions.NotRequired[CatalogQueryItemsForItemOptionsParams]
    """
    A query expression to return items that contains the specified item options (as identified the corresponding `CatalogItemOption` IDs).
    """

    item_variations_for_item_option_values_query: typing_extensions.NotRequired[
        CatalogQueryItemVariationsForItemOptionValuesParams
    ]
    """
    A query expression to return item variations (of the [CatalogItemVariation](entity:CatalogItemVariation) type) that
    contain all of the specified `CatalogItemOption` IDs.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class OrderPricingOptions(UncheckedBaseModel):
    """
    Pricing options for an order. The options affect how the order's price is calculated.
    They can be used, for example, to apply automatic price adjustments that are based on preconfigured
    [pricing rules](entity:CatalogPricingRule).
    """

    auto_apply_discounts: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The option to determine whether pricing rule-based
    discounts are automatically applied to an order.
    """

    auto_apply_taxes: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The option to determine whether rule-based taxes are automatically
    applied to an order when the criteria of the corresponding rules are met.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawMerchantsClient
from .custom_attribute_definitions.client import CustomAttributeDefinitionsClient
from .custom_attributes.client import CustomAttributesClient
import typing
from ..core.request_options import RequestOptions
from ..core.pagination import SyncPager
from ..types.merchant import Merchant
from ..types.list_merchants_response import ListMerchantsResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.get_merchant_response import GetMerchantResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawMerchantsClient
from .custom_attribute_definitions.client import AsyncCustomAttributeDefinitionsClient
from .custom_attributes.client import AsyncCustomAttributesClient
from ..core.pagination import AsyncPager


class MerchantsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMerchantsClient(client_wrapper=client_wrapper)
        self.custom_attribute_definitions = CustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

        self.custom_attributes = CustomAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawMerchantsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMerchantsClient
        """
        return self._raw_client

    def list(
        self, *, cursor: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> SyncPager[Merchant]:
        """
        Provides details about the merchant associated with a given access token.

        The access token used to connect your application to a Square seller is associated
        with a single merchant. That means that `ListMerchants` returns a list
        with a single `Merchant` object. You can specify your personal access token
        to get your own merchant information or specify an OAuth token to get the
        information for the merchant that granted your application access.

        If you know the merchant ID, you can also use the [RetrieveMerchant](api-endpoint:Merchants-RetrieveMerchant)
        endpoint to retrieve the merchant information.

        Parameters
        ----------
        cursor : typing.Optional[int]
            The cursor generated by the previous response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Merchant]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.merchants.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/merchants",
            method="GET",
            params={
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListMerchantsResponse,
                    construct_type(
                        type_=ListMerchantsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.merchant
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, merchant_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetMerchantResponse:
        """
        Retrieves the `Merchant` object for the given `merchant_id`.

        Parameters
        ----------
        merchant_id : str
            The ID of the merchant to retrieve. If the string "me" is supplied as the ID,
            then retrieve the merchant that is currently accessible to this call.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetMerchantResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.merchants.get(
            merchant_id="merchant_id",
        )
        """
        response = self._raw_client.get(merchant_id, request_options=request_options)
        return response.data


class AsyncMerchantsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMerchantsClient(client_wrapper=client_wrapper)
        self.custom_attribute_definitions = AsyncCustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

        self.custom_attributes = AsyncCustomAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawMerchantsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMerchantsClient
        """
        return self._raw_client

    async def list(
        self, *, cursor: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncPager[Merchant]:
        """
        Provides details about the merchant associated with a given access token.

        The access token used to connect your application to a Square seller is associated
        with a single merchant. That means that `ListMerchants` returns a list
        with a single `Merchant` object. You can specify your personal access token
        to get your own merchant information or specify an OAuth token to get the
        information for the merchant that granted your application access.

        If you know the merchant ID, you can also use the [RetrieveMerchant](api-endpoint:Merchants-RetrieveMerchant)
        endpoint to retrieve the merchant information.

        Parameters
        ----------
        cursor : typing.Optional[int]
            The cursor generated by the previous response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Merchant]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.merchants.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/merchants",
            method="GET",
            params={
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListMerchantsResponse,
                    construct_type(
                        type_=ListMerchantsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.merchant
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, merchant_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetMerchantResponse:
        """
        Retrieves the `Merchant` object for the given `merchant_id`.

        Parameters
        ----------
        merchant_id : str
            The ID of the merchant to retrieve. If the string "me" is supplied as the ID,
            then retrieve the merchant that is currently accessible to this call.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetMerchantResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.merchants.get(
                merchant_id="merchant_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(merchant_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .cash_drawer_shift import CashDrawerShiftParams
import typing
from .error import ErrorParams


class GetCashDrawerShiftResponseParams(typing_extensions.TypedDict):
    cash_drawer_shift: typing_extensions.NotRequired[CashDrawerShiftParams]
    """
    The cash drawer shift queried for.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class MeasurementUnitCustom(UncheckedBaseModel):
    """
    The information needed to define a custom unit, provided by the seller.
    """

    name: str = pydantic.Field()
    """
    The name of the custom unit, for example "bushel".
    """

    abbreviation: str = pydantic.Field()
    """
    The abbreviation of the custom unit, such as "bsh" (bushel). This appears
    in the cart for the Point of Sale app, and in reports.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CatalogCustomAttributeDefinitionStringConfigParams(typing_extensions.TypedDict):
    """
    Configuration associated with Custom Attribute Definitions of type `STRING`.
    """

    enforce_uniqueness: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    If true, each Custom Attribute instance associated with this Custom Attribute
    Definition must have a unique value within the seller's catalog. For
    example, this may be used for a value like a SKU that should not be
    duplicated within a seller's catalog. May not be modified after the
    definition has been created.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
import typing
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.get_merchant_response import GetMerchantResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse


class RawMerchantsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, merchant_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetMerchantResponse]:
        """
        Retrieves the `Merchant` object for the given `merchant_id`.

        Parameters
        ----------
        merchant_id : str
            The ID of the merchant to retrieve. If the string "me" is supplied as the ID,
            then retrieve the merchant that is currently accessible to this call.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetMerchantResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/merchants/{jsonable_encoder(merchant_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetMerchantResponse,
                    construct_type(
                        type_=GetMerchantResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawMerchantsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, merchant_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetMerchantResponse]:
        """
        Retrieves the `Merchant` object for the given `merchant_id`.

        Parameters
        ----------
        merchant_id : str
            The ID of the merchant to retrieve. If the string "me" is supplied as the ID,
            then retrieve the merchant that is currently accessible to this call.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetMerchantResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/merchants/{jsonable_encoder(merchant_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetMerchantResponse,
                    construct_type(
                        type_=GetMerchantResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

GiftCardActivityType = typing.Union[
    typing.Literal[
        "ACTIVATE",
        "LOAD",
        "REDEEM",
        "CLEAR_BALANCE",
        "DEACTIVATE",
        "ADJUST_INCREMENT",
        "ADJUST_DECREMENT",
        "REFUND",
        "UNLINKED_ACTIVITY_REFUND",
        "IMPORT",
        "BLOCK",
        "UNBLOCK",
        "IMPORT_REVERSAL",
        "TRANSFER_BALANCE_FROM",
        "TRANSFER_BALANCE_TO",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.inventory_state import InventoryState


class InventoryAdjustmentGroupParams(typing_extensions.TypedDict):
    id: typing_extensions.NotRequired[str]
    """
    A unique ID generated by Square for the
    `InventoryAdjustmentGroup`.
    """

    root_adjustment_id: typing_extensions.NotRequired[str]
    """
    The inventory adjustment of the composed variation.
    """

    from_state: typing_extensions.NotRequired[InventoryState]
    """
    Representative `from_state` for adjustments within the group. For example, for a group adjustment from `IN_STOCK` to `SOLD`,
    there can be two component adjustments in the group: one from `IN_STOCK`to `COMPOSED` and the other one from `COMPOSED` to `SOLD`.
    Here, the representative `from_state` for the `InventoryAdjustmentGroup` is `IN_STOCK`.
    See [InventoryState](#type-inventorystate) for possible values
    """

    to_state: typing_extensions.NotRequired[InventoryState]
    """
    Representative `to_state` for adjustments within group. For example, for a group adjustment from `IN_STOCK` to `SOLD`,
    the two component adjustments in the group can be from `IN_STOCK` to `COMPOSED` and from `COMPOSED` to `SOLD`.
    Here, the representative `to_state` of the `InventoryAdjustmentGroup` is `SOLD`.
    See [InventoryState](#type-inventorystate) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class OrderPricingOptionsParams(typing_extensions.TypedDict):
    """
    Pricing options for an order. The options affect how the order's price is calculated.
    They can be used, for example, to apply automatic price adjustments that are based on preconfigured
    [pricing rules](entity:CatalogPricingRule).
    """

    auto_apply_discounts: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    The option to determine whether pricing rule-based
    discounts are automatically applied to an order.
    """

    auto_apply_taxes: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    The option to determine whether rule-based taxes are automatically
    applied to an order when the criteria of the corresponding rules are met.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Phase(UncheckedBaseModel):
    """
    Represents a phase, which can override subscription phases as defined by plan_id
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    id of subscription phase
    """

    ordinal: typing.Optional[int] = pydantic.Field(default=None)
    """
    index of phase in total subscription plan
    """

    order_template_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    id of order to be used in billing
    """

    plan_phase_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    the uid from the plan's phase in catalog
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .checkout_merchant_settings_payment_methods_afterpay_clearpay_eligibility_range import (
    CheckoutMerchantSettingsPaymentMethodsAfterpayClearpayEligibilityRange,
)
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CheckoutMerchantSettingsPaymentMethodsAfterpayClearpay(UncheckedBaseModel):
    """
    The settings allowed for AfterpayClearpay.
    """

    order_eligibility_range: typing.Optional[CheckoutMerchantSettingsPaymentMethodsAfterpayClearpayEligibilityRange] = (
        pydantic.Field(default=None)
    )
    """
    Afterpay is shown as an option for order totals falling within the configured range.
    """

    item_eligibility_range: typing.Optional[CheckoutMerchantSettingsPaymentMethodsAfterpayClearpayEligibilityRange] = (
        pydantic.Field(default=None)
    )
    """
    Afterpay is shown as an option for item totals falling within the configured range.
    """

    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the payment method is enabled for the account.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .device_component_details_measurement import DeviceComponentDetailsMeasurementParams


class DeviceComponentDetailsWiFiDetailsParams(typing_extensions.TypedDict):
    active: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    A boolean to represent whether the WiFI interface is currently active.
    """

    ssid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the connected WIFI network.
    """

    ip_address_v4: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The string representation of the device’s IPv4 address.
    """

    secure_connection: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The security protocol for a secure connection (e.g. WPA2). None provided if the connection
    is unsecured.
    """

    signal_strength: typing_extensions.NotRequired[DeviceComponentDetailsMeasurementParams]
    """
    A representation of signal strength of the WIFI network connection.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .money import Money
import pydantic
from .loyalty_program_accrual_rule_tax_mode import LoyaltyProgramAccrualRuleTaxMode
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyProgramAccrualRuleVisitData(UncheckedBaseModel):
    """
    Represents additional data for rules with the `VISIT` accrual type.
    """

    minimum_amount_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The minimum purchase required during the visit to quality for points.
    """

    tax_mode: LoyaltyProgramAccrualRuleTaxMode = pydantic.Field()
    """
    Indicates how taxes should be treated when calculating the purchase amount to determine whether the visit qualifies for points. 
    This setting applies only if `minimum_amount_money` is specified.
    See [LoyaltyProgramAccrualRuleTaxMode](#type-loyaltyprogramaccrualruletaxmode) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .card import CardParams
from .device_details import DeviceDetailsParams
from .card_payment_timeline import CardPaymentTimelineParams
from .error import ErrorParams


class CardPaymentDetailsParams(typing_extensions.TypedDict):
    """
    Reflects the current status of a card payment. Contains only non-confidential information.
    """

    status: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The card payment's current state. The state can be AUTHORIZED, CAPTURED, VOIDED, or
    FAILED.
    """

    card: typing_extensions.NotRequired[CardParams]
    """
    The credit card's non-confidential details.
    """

    entry_method: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The method used to enter the card's details for the payment. The method can be
    `KEYED`, `SWIPED`, `EMV`, `ON_FILE`, or `CONTACTLESS`.
    """

    cvv_status: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The status code returned from the Card Verification Value (CVV) check. The code can be
    `CVV_ACCEPTED`, `CVV_REJECTED`, or `CVV_NOT_CHECKED`.
    """

    avs_status: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The status code returned from the Address Verification System (AVS) check. The code can be
    `AVS_ACCEPTED`, `AVS_REJECTED`, or `AVS_NOT_CHECKED`.
    """

    auth_result_code: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The status code returned by the card issuer that describes the payment's
    authorization status.
    """

    application_identifier: typing_extensions.NotRequired[typing.Optional[str]]
    """
    For EMV payments, the application ID identifies the EMV application used for the payment.
    """

    application_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    For EMV payments, the human-readable name of the EMV application used for the payment.
    """

    application_cryptogram: typing_extensions.NotRequired[typing.Optional[str]]
    """
    For EMV payments, the cryptogram generated for the payment.
    """

    verification_method: typing_extensions.NotRequired[typing.Optional[str]]
    """
    For EMV payments, the method used to verify the cardholder's identity. The method can be
    `PIN`, `SIGNATURE`, `PIN_AND_SIGNATURE`, `ON_DEVICE`, or `NONE`.
    """

    verification_results: typing_extensions.NotRequired[typing.Optional[str]]
    """
    For EMV payments, the results of the cardholder verification. The result can be
    `SUCCESS`, `FAILURE`, or `UNKNOWN`.
    """

    statement_description: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The statement description sent to the card networks.
    
    Note: The actual statement description varies and is likely to be truncated and appended with
    additional information on a per issuer basis.
    """

    device_details: typing_extensions.NotRequired[DeviceDetailsParams]
    """
    __Deprecated__: Use `Payment.device_details` instead.
    
    Details about the device that took the payment.
    """

    card_payment_timeline: typing_extensions.NotRequired[CardPaymentTimelineParams]
    """
    The timeline for card payments.
    """

    refund_requires_card_presence: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Whether the card must be physically present for the payment to
    be refunded.  If set to `true`, the card must be present.
    """

    errors: typing_extensions.NotRequired[typing.Optional[typing.Sequence[ErrorParams]]]
    """
    Information about errors encountered during the request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .location import Location
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CreateLocationResponse(UncheckedBaseModel):
    """
    The response object returned by the [CreateLocation](api-endpoint:Locations-CreateLocation) endpoint.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Information about [errors](https://developer.squareup.com/docs/build-basics/handling-errors) encountered during the request.
    """

    location: typing.Optional[Location] = pydantic.Field(default=None)
    """
    The newly created `Location` object.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.day_of_week import DayOfWeek
import typing


class BusinessHoursPeriodParams(typing_extensions.TypedDict):
    """
    Represents a period of time during which a business location is open.
    """

    day_of_week: typing_extensions.NotRequired[DayOfWeek]
    """
    The day of the week for this time period.
    See [DayOfWeek](#type-dayofweek) for possible values
    """

    start_local_time: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The start time of a business hours period, specified in local time using partial-time
    RFC 3339 format. For example, `8:30:00` for a period starting at 8:30 in the morning.
    Note that the seconds value is always :00, but it is appended for conformance to the RFC.
    """

    end_local_time: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The end time of a business hours period, specified in local time using partial-time
    RFC 3339 format. For example, `21:00:00` for a period ending at 9:00 in the evening.
    Note that the seconds value is always :00, but it is appended for conformance to the RFC.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .money import MoneyParams
import typing_extensions
import typing
from ..types.gift_card_activity_redeem_status import GiftCardActivityRedeemStatus


class GiftCardActivityRedeemParams(typing_extensions.TypedDict):
    """
    Represents details about a `REDEEM` [gift card activity type](entity:GiftCardActivityType).
    """

    amount_money: MoneyParams
    """
    The amount deducted from the gift card for the redemption. This value is a positive integer.
    
    Applications that use a custom payment processing system must specify this amount in the 
    [CreateGiftCardActivity](api-endpoint:GiftCardActivities-CreateGiftCardActivity) request.
    """

    payment_id: typing_extensions.NotRequired[str]
    """
    The ID of the payment that represents the gift card redemption. Square populates this field 
    if the payment was processed by Square.
    """

    reference_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A client-specified ID that associates the gift card activity with an entity in another system. 
    
    Applications that use a custom payment processing system can use this field to track information
    related to an order or payment.
    """

    status: typing_extensions.NotRequired[GiftCardActivityRedeemStatus]
    """
    The status of the gift card redemption. Gift cards redeemed from Square Point of Sale or the 
    Square Seller Dashboard use a two-state process: `PENDING` 
    to `COMPLETED` or `PENDING` to  `CANCELED`. Gift cards redeemed using the Gift Card Activities API 
    always have a `COMPLETED` status.
    See [Status](#type-status) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .customer_group import CustomerGroupParams


class ListCustomerGroupsResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the [ListCustomerGroups](api-endpoint:CustomerGroups-ListCustomerGroups) endpoint.

    Either `errors` or `groups` is present in a given response (never both).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    groups: typing_extensions.NotRequired[typing.Sequence[CustomerGroupParams]]
    """
    A list of customer groups belonging to the current seller.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    A pagination cursor to retrieve the next set of results for your
    original query to the endpoint. This value is present only if the request
    succeeded and additional results are available.
    
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.catalog_item_food_and_beverage_details_dietary_preference_type import (
    CatalogItemFoodAndBeverageDetailsDietaryPreferenceType,
)
from ..types.catalog_item_food_and_beverage_details_dietary_preference_standard_dietary_preference import (
    CatalogItemFoodAndBeverageDetailsDietaryPreferenceStandardDietaryPreference,
)
import typing


class CatalogItemFoodAndBeverageDetailsDietaryPreferenceParams(typing_extensions.TypedDict):
    """
    Dietary preferences that can be assigned to an `FOOD_AND_BEV` item and its ingredients.
    """

    type: typing_extensions.NotRequired[CatalogItemFoodAndBeverageDetailsDietaryPreferenceType]
    """
    The dietary preference type. Supported values include `STANDARD` and `CUSTOM` as specified in `FoodAndBeverageDetails.DietaryPreferenceType`.
    See [DietaryPreferenceType](#type-dietarypreferencetype) for possible values
    """

    standard_name: typing_extensions.NotRequired[
        CatalogItemFoodAndBeverageDetailsDietaryPreferenceStandardDietaryPreference
    ]
    """
    The name of the dietary preference from a standard pre-defined list. This should be null if it's a custom dietary preference.
    See [StandardDietaryPreference](#type-standarddietarypreference) for possible values
    """

    custom_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of a user-defined custom dietary preference. This should be null if it's a standard dietary preference.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .bank_account import BankAccountParams


class GetBankAccountResponseParams(typing_extensions.TypedDict):
    """
    Response object returned by `GetBankAccount`.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information on errors encountered during the request.
    """

    bank_account: typing_extensions.NotRequired[BankAccountParams]
    """
    The requested `BankAccount` object.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams


class OrderReturnTipParams(typing_extensions.TypedDict):
    """
    A tip being returned.
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID that identifies the tip only within this order.
    """

    applied_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of tip being returned
    --
    """

    source_tender_uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The tender `uid` from the order that contains the original application of this tip.
    """

    source_tender_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The tender `id` from the order that contains the original application of this tip.
    """
